% Fleur ter Huurne  | 4 august 2015
% NDJF circuit - fig 7A Boareto - 20 interacting cells
% Edited by Sandra Loerakker, 5 July 2016
%   - Added comments
% Edited by Sandra Loerakker, 8 September 2016
%   - Added comments
% Edited by Sandra Loerakker, 9 December 2016
%   - Removed separation of Ne (increase similarity with equations in paper)

clearvars
close all
clc

% ------------------------------------------------------------------------
% constants
gamma = 0.1;    % [1/h] degradation of Notch (N), Delta (D), Jagged (J)
gammaI = 0.5;   % [1/h] degradation of NICD (I)
kc = 5e-04;     % [1/h] strength cis-inhibition
kt = 2.5e-05;   % [1/h] strengtth transactivation
Npr = 1400;     % [number of proteins] production rate Notch in molec
Dpr = 100;      % [number of proteins] production rate Delta in molec
Jpr = 1600;     % [number of proteins] production rate Delta in molec
I0 = 200;       % [number of proteins] threshold of the Hill function in molec
lN = 2.0;       % Lambda Notch of shifted Hill function [dimensionless]
lD = 0.0;       % Lambda Delta of shifted Hill function [dimensionless]
lJ = 2.0;       % Lambda Jagged of shifted Hill function [dimensionless]
nN = 2.0;       % Hill Notch coefficients [dimensionless]
nD = 2.0;       % Hill Delta coefficients [dimensionless]
nJ = 5.0;       % Hill Jagged coefficients [dimensionless]

nf = 1.0;
lDf = 1.0;      % Lambda Delta of shifted Hill function [dimensionless]
lJf = 1.0;      % Lambda Jagged of shifted Hill function [dimensionless]

Jpr_EC = 4000;

eps_ref = 0.075;
sig_ref = 107;
cell_size = 0.01; % mm
radius = 3.2; % mm


% -------------------------------------------------------------------------
% variables 
tmin = 0;  % start time
tmax = 250; % end time
dt = 0.01; % time step (dt)
% ncells = 5; % number of cells 
ncombs = 25; % number of simulations - take average over 100 simulations

% -------------------------------------------------------------------------
% Dpr_range = 900:300:1800;
ncells_range = [1:100];

% N_total = zeros(length(ncells_range),ncells_range(end));
% D_total = zeros(length(ncells_range),ncells_range(end));
% J_total = zeros(length(ncells_range),ncells_range(end));
% I_total = zeros(length(ncells_range),ncells_range(end));

% Save number of states per length (multiple simulations)
nS = zeros(ncombs,1);
nSR = zeros(ncombs,1);
nR = zeros(ncombs,1);

% Save number of cell states and average protein levels for different lengths
ncellstates1 = zeros(length(ncells_range),4);
proteins_levels1 = zeros(length(ncells_range),8);

% Save average protein levels per cell for different lengths
N_mean = zeros(length(ncells_range),ncells_range(end));
D_mean = zeros(length(ncells_range),ncells_range(end));
J_mean = zeros(length(ncells_range),ncells_range(end));
I_mean = zeros(length(ncells_range),ncells_range(end));
I_end = zeros(length(ncells_range),ncells_range(end));

for ppp = 1:length(ncells_range)
    ncells = ncells_range(ppp)
    
    % Mechanosensitivity
    A_Notch = -5.79;
    A_Jagged = -4.17;
    thickness = ncells*cell_size;
    sigma = 16*radius/thickness;
    Nmf = exp(A_Notch*(eps_ref/sig_ref)*sigma);
    Jmf = exp(A_Jagged*(eps_ref/sig_ref)*sigma);
    
    t = zeros((tmax/dt),1);
    t(1) = tmin;
    
    % array with protein numbers in each cell at each time point 
    Notch = zeros((tmax/dt),ncells);
    Delta = zeros((tmax/dt),ncells);
    Jagged = zeros((tmax/dt),ncells);
    NICD = zeros((tmax/dt),ncells);
    
    % array with protein numbers in each cell at t = tmax for all
    N_combs = zeros(ncombs,ncells);
    D_combs = zeros(ncombs,ncells);
    J_combs = zeros(ncombs,ncells);
    I_combs = zeros(ncombs,ncells);
        
    for s = 1:ncombs
        % random initial condition for each cell, chosen between a certain
        % range (based on bifurgation plots Boareto)
        % same ranges as Boareto used were chosen
        rng('shuffle') % still random after restarting Matlab
        for c_int = 1:ncells
            % value for N, D, J and I at tmin (initial value)
            Notch(1,c_int) = randi([0, 6000],1,1);
            Delta(1,c_int) = randi([0, 6000],1,1);
            Jagged(1,c_int) = randi([0, 6000],1,1);
            NICD(1,c_int) = randi([0, 600],1,1);
        end
        
        i = 0;
        for time = tmin+dt:dt:tmax
            i = i + 1;
            % calculate Ne, De and Je for each cell
            % function Hillfunction = HS(I,I0,n,l)
            % function Neighbors = extp_dt(ncells,Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i)
            Neighbors = extp_F_dt_oneway(ncells,Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i,time,tmin,tmax,dt,Jpr_EC);
            NeD = Neighbors(1,:); % number of Notch molecules in neighboring cells
            NeJ = Neighbors(2,:); % number of Notch molecules in neighboring cells
            De = Neighbors(3,:); % number of Delta molecules in neighboring cells 
            Je = Neighbors(4,:); % number of Jagged molecules in neighboring cells
            
            t(i+1) = time;
            % calculate number of proteins (N,D,J,I) at each time point
            % expicit solution t_new = t_old + differential equations
            % store number of proteins at the next time step in the next row
            Notch(i+1,:) = Notch(i,:) + dt*(Npr*Nmf*HS(NICD(i,:),I0,nN,lN) - Notch(i,:).*((kc*Delta(i,:) + kt*De) ...
                .*HS(NICD(i,:),I0,nf,lDf) + (kc*Jagged(i,:) + kt*Je).*HS(NICD(i,:),I0,nf,lJf)) - gamma*Notch(i,:));
            Delta(i+1,:) = Delta(i,:) + dt*(Dpr*HS(NICD(i,:),I0,nD,lD) - Delta(i,:).*(kc*Notch(i,:) + kt*NeD) ...
                .*HS(NICD(i,:),I0,nf,lDf) - gamma*Delta(i,:));
            Jagged(i+1,:) = Jagged(i,:) + dt*(Jpr*Jmf*HS(NICD(i,:),I0,nJ,lJ) - Jagged(i,:).*(kc*Notch(i,:) + kt*NeJ) ...
                .*HS(NICD(i,:),I0,nf,lJf) - gamma*Jagged(i,:));
            NICD(i+1,:) = NICD(i,:) + dt*(kt*Notch(i,:).*(De.*HS(NICD(i,:),I0,nf,lDf) + Je.*HS(NICD(i,:),I0,nf,lJf)) ...
                - gammaI.*NICD(i,:));
            for r = 1:ncells
                if Notch(i+1,r) < 0
                    Notch(i+1,r) = 0;
                end
                if Delta(i+1,r) < 0
                    Delta(i+1,r) = 0;
                end
                if Jagged(i+1,r) < 0
                    Jagged(i+1,r) = 0;
                end
                if NICD(i+1,r) < 0
                    NICD(i+1,r) = 0;
                end
            end
        end
        
%         if ncells == ncells_range(end)
%             figure; hold on; plot(t,Notch(:,[1,10,50,ncells])); title('Notch');
%             figure; hold on; plot(t,Delta(:,[1,10,50,ncells])); title('Delta');
%             figure; hold on; plot(t,Jagged(:,[1,10,50,ncells])); title('Jagged');
%             figure; hold on; plot(t,NICD(:,[1,10,50,ncells])); title('NICD');
%         end 
        
        % save for each simulation the value of N, D, J and I at time = tmax
        % s is simulation number (s/ncombs)
        N_combs(s,:) = Notch(end,:);
        D_combs(s,:) = Delta(end,:);
        J_combs(s,:) = Jagged(end,:);
        I_combs(s,:) = NICD(end,:);
        
    end % end for s = 1:ncombs
    
    % store average protein levels per cell for each length
    N_mean(ppp,1:ncells) = mean(N_combs,1);
    D_mean(ppp,1:ncells) = mean(D_combs,1);
    J_mean(ppp,1:ncells) = mean(J_combs,1);
    I_mean(ppp,1:ncells) = mean(I_combs,1);
    I_end(ppp,1:ncells) = I_combs(end,:);
    
    % STD?? N_std(ppp,1:cells) = std(N_combs,1)
    N_std(ppp,1:ncells) = std(N_combs);
    D_std(ppp,1:ncells) = std(D_combs);
    J_std(ppp,1:ncells) = std(J_combs);
    I_std(ppp,1:ncells) = std(I_combs);
    
    % store average + standard deviation protein level per length
    proteins_levels1(ppp,1) = ncells;
    proteins_levels1(ppp,2) = mean(mean(N_combs));
    proteins_levels1(ppp,3) = mean(std(N_combs));
    proteins_levels1(ppp,4) = mean(mean(D_combs));
    proteins_levels1(ppp,5) = mean(std(D_combs));
    proteins_levels1(ppp,6) = mean(mean(J_combs));
    proteins_levels1(ppp,7) = mean(std(J_combs));
    proteins_levels1(ppp,8) = mean(mean(I_combs));
    proteins_levels1(ppp,9) = mean(std(I_combs));
    
    % determine cell fate based on average NICD value
    cstate = zeros(ncombs,ncells);
    for q = 1:size(I_combs,1)
        for w = 1:size(I_combs,2)
            if I_combs(q,w) < 100 % undifferentiated (default Fleur: 145)
                cstate(q,w) = 0;
            elseif I_combs(q,w) > 300 % (default Fleur: 300)
                cstate(q,w) = 1;
            else % differentiated
                cstate(q,w) = 0.5;
            end
        end
    end
    
    % calculate number of differentiated and undifferentiated cells for each simulation
    for z = 1:ncombs 
        v = cstate(z,:);
        nS(z) = length(find(v == 0));
        nR(z) = length(find(v == 1));
        nSR(z) = length(find(v == 0.5));
    end
    
    % Calculate the average and std of the number of states for all simulations
    mean_nS = mean(nS);
    mean_nSR = mean(nSR);
    mean_nR = mean(nR);
    std_nS = std(nS);
    std_nSR = std(nSR);
    std_nR = std(nR);
    
    % mean values of ncombs
    ncellstates1(ppp,1) = ncells;
%     ncellstates1(ppp,2) = h;
%     ncellstates1(ppp,3) = T;
%     ncellstates1(ppp,4) = length(nS);
%     ncellstates1(ppp,5) = length(nSR);
%     ncellstates1(ppp,6) = length(nR);
%     ncellstates1(ppp,7) = length(nS)/ncells*100;
%     ncellstates1(ppp,8) = length(nSR)/ncells*100;
%     ncellstates1(ppp,9) = length(nR)/ncells*100;
    ncellstates1(ppp,4) = mean_nS;
    ncellstates1(ppp,5) = mean_nSR;
    ncellstates1(ppp,6) = mean_nR;
    ncellstates1(ppp,7) = std_nS;
    ncellstates1(ppp,8) = std_nSR;
    ncellstates1(ppp,9) = std_nR;    
    ncellstates1(ppp,10) = mean_nS/ncells*100;
    ncellstates1(ppp,11) = mean_nSR/ncells*100;
    ncellstates1(ppp,12) = mean_nR/ncells*100;
    ncellstates1(ppp,13) = std_nS/ncells*100;
    ncellstates1(ppp,14) = std_nSR/ncells*100;
    ncellstates1(ppp,15) = std_nR/ncells*100;
   
end % end for r = 1:length(Dpr_range)

% Calculate transition thickness
nSR_tot = ncellstates1(:,11);
nSR_diff = zeros(size(nSR_tot,1),1);
for i = 2:length(nSR_diff)
    nSR_diff(i) = nSR_tot(i) - nSR_tot(i-1);
end
i_trans = find(nSR_diff == max(nSR_diff))
t_trans = (i_trans + 1)*cell_size

save('wksp_original_1D.mat');